/*************************************************************/
/*                                                           */
/*                         TXANAL.C                          */
/*                Text Readability Analyzer                  */
/*                                                           */
/*                  written for Lattice C                    */
/*                     by Thomas Castle                      */   
/*************************************************************/

/*************************************************************/
/* Lattice C complier command line:    lc -n -cw {filename}  */
/* GST Linker:  link {filename} -with CGEM -nolist -debug    */
/*************************************************************/
            
#include <ctype.h>     /* for the isalpha().. */
#include <stdio.h>     /* for the string functions */
#include <portab.h>    /* for you Alcyon & Megamaxers */
#include <gemlib.h>    /* for the gem functions */
#define  TXMENU   0    /* menutree from TXANAL.H */
#define  ABOUT    8    /* STRING in tree TXMENU */
#define  ANALYZE 17    /* STRING in tree TXMENU */
#define  QUIT    19    /* STRING in tree TXMENU */
#define  NOQUIT  20    /* STRING in tree TXMENU */

/****************************************************************/
/*                            GLOBALS                           */
/****************************************************************/
                 
FILE   *fp;    /* Alcyon uses file handles (int) instead of file pointers */
char   wi_name[] = "Text Readability Analyzer";
char   filename[] = "\0";
char   filepath[80];     /* The portab.h file converts WORD to short int */
char   string[10];       /* since Lattice uses 32-bit integers for int and */
char   buf[256];         /* long.  The short int is 16-bits.             */         
OBJECT *menutree;
WORD   ap_id;
WORD   work_in[12],work_out[57];
WORD   handle,xmax,ymax,wmax,hmax;
WORD   gr_1,gr_2,gr_3,gr_4;
WORD   wi_1,wi_2,wi_3,wi_4;
WORD   w_handle;
WORD   mgbuf[8];
WORD   wi_type;
WORD   clip[4];
WORD   dflag,xt;
WORD   rez;
void   bye(),loop(),display_stats(),redraw(),wi_fill(),trunc_ext();
void   put_labels(),process(),get_level();
LONG   parse(), div_round(); 
char alert0[] = "[2][Analysis complete ][ OK ]";
char alert1[] = "[3][Can't find TXANAL.RSC file ][ OK ]";
char alert2[] = "[3][You must change to |    Medium Resolution][ OK ]";
char alert3[] = "[0][Text Readability Analyzer |    by Thomas Castle | ][ OK ]";
char alert4[] = "[3][We're having trouble | opening your file ][ OK ]";
char flesch5[] =  "        Grade 5         ";
char flesch6[] =  "        Grade 6         ";
char flesch7[] =  "        Grade 7         ";
char flesch89[] = "  Grade 8-9 (Jr. High)  ";
char flesch10[] = " Grade 10-12 (Sr. High) ";
char fleschc[] =  "     College student    ";
char fleschg[] =  "    College graduate    ";



/*************************************************************************/
/*                                MAIN                                   */
/*************************************************************************/

void
main()
{
        if(initialize()){
            loop();
            bye();
        }  
}


/*************************************************************************/
/*                       WINDOW CREATION & MANAGEMENT                    */
/*                                                                       */
/* Tidbits of this were modelled from the EGAES.C example file included  */
/* with the Lattice C complier, the DEMO.C file of Tim Oren and Tom      */
/* Rolander, and from Abacus' Atari ST GEM Programmer's Reference.       */
/*************************************************************************/    

initialize()
{
   WORD i;
   
   ap_id = appl_init();
   handle=graf_handle(&gr_1,&gr_2,&gr_3,&gr_4);

   for(i=0; i<10; i++){
      work_in[i] = 1;
   }
   work_in[10] = 2;

   v_opnvwk(work_in, &handle, work_out);

   if(!rsrc_load("txanal.rsc")){
        form_alert(1,alert1);
        exit(1);           
   }                       

   if((rez = xbios(0x04)) == 0){     /* This is the Alcyon Getrez() */
         form_alert(1,alert2);
         exit(1);
   }     

   rsrc_gaddr(0,TXMENU,&menutree); 
   menu_bar(menutree,1);

   graf_mouse(0,0x0L);
   wi_type=0x0001;
/* hide the mouse. Could use graf_mouse(M_OFF,0x0L); but this is cleaner. */
   lineaa();   


/* request size of desktop window */
   wind_get(0,4,&wi_1,&wi_2,&wi_3,&wi_4);

/* calculate size of work area */
   wind_calc(1,wi_type,wi_1,wi_2,wi_3,wi_4,&xmax,&ymax,&wmax,&hmax);

/* make window of the max size */
   w_handle=wind_create(wi_type,wi_1,wi_2,wi_3,wi_4);
   wind_set(w_handle,2,ADDR( wi_name ),0,0);
   wind_open(w_handle,wi_1,wi_2,wi_3,wi_4);
   clip[0]=xmax;
   clip[1]=ymax;
   clip[2]=xmax+wmax-1;
   clip[3]=ymax+hmax-1;
   wi_fill();
   linea9();  /* show the mouse */
   return(1);
}


void
wi_fill()
{
   vs_clip(handle,1,clip);  
   vsf_interior(handle,2);  
   vsf_style(handle,4);     
   vsf_color(handle,1);     
   v_bar(handle,clip);      
   vs_clip(handle,0,clip);  
}


void
redraw(x,y,w,h)
WORD x,y,w,h;
{
         WORD wx,wy,ww,wh; 

         v_hide_c(handle);
         wind_update(1);
         wind_get(w_handle,WF_FIRSTXYWH, &wx,&wy,&ww,&wh);
         while(ww && wh)
         {
             if(x<wx+ww && wx<x+w && y<wy+wh && wy<y+h)
                  {    
                  clip[0]=wx; clip[1]=wy;
                  clip[2]=wx+ww-1; clip[3]=wy+wh-1;
                  vs_clip(handle,1,clip);
                  wi_fill();
             }   /* end of if... */
             wind_get(w_handle,WF_NEXTXYWH, &wx,&wy,&ww,&wh);
         }  /* end of while .. */
         wind_update(0);
         v_show_c(handle);  
         vs_clip(handle,0,clip);
}


void
bye()
{
   wind_close(w_handle);
   wind_delete(w_handle);
   v_clsvwk(handle);
   appl_exit();
   _exit(0);
}


/*********************************************************************/
/*                         EVENT_MULTI LOOP                          */
/*********************************************************************/
void
loop()
{
      WORD event, trash, fbutton, exitflag;
      char path[64], bigpath[80];

      exitflag = 0;
      do{
        event = evnt_multi(MU_MESAG, /* ignore mouse button */
                                     /* and keyboard events */
                1,1,1,0,0,0,0,0,0,0,0,0,0,
                &mgbuf,0,0,
                &trash,&trash,&trash,&trash,&trash,&trash);
      

         if(event & MU_MESAG)
         {
             switch(mgbuf[0]){
                 case WM_REDRAW :
                      redraw(mgbuf[4],mgbuf[5],mgbuf[6],mgbuf[7]);
                      break;
                 case MN_SELECTED :
                      menu_tnormal(menutree,mgbuf[3],1);
                      switch(mgbuf[4]){          /* the menu item */
                          case ABOUT :
                               form_alert(1,alert3); 
                               break;
                          case ANALYZE :
                               path[0] = gemdos(0x19) + 'A';  /* Alcyon's  */ 
                               path[1] = ':';                 /* Dgetdrv() */ 
                               gemdos(0x47,&path[2],0);  /* and Dgetpath() */
                               strcpy(bigpath,path);
                               strcat(bigpath,"\\*.*"); 
                               if(fsel_input(bigpath,filename,&fbutton)){
                                  if(fbutton){
                                      strcpy(filepath,bigpath);
                                      trunc_ext();
                                      strcat(filepath,filename);   
                                      if(openfile()){
                                          gemdos(0x0e,filepath[0]-'A');
                                          gemdos(0x3b,&filepath[2]); 
                                /* the Alcyon Dsetdrv() and Dsetpath().. */
                                          redraw(xmax,ymax,wmax,hmax);
                                          put_labels();
                                          process();
                                          appl_read(ap_id,16,mgbuf); 
                               /* to prevent a redraw at end of process() */
                                      }    
                                  } /* end of if... */
                                }        /* end of if... */
                                break; 
                          case QUIT :
                                exitflag = 1;
                                break;
                          case NOQUIT:
                                break;
                      }  /* end of switch...  */
             }   /* end of switch ... */
        }     /* end of if...  */      
   } while(!exitflag);      /* end of do... */
}                              /* end of loop() ... */



/************************************************************************/
/*                          FILE HANDLING                               */
/*                                                                      */
/* The fsel_input() function is used to select files.  Any drive can    */
/* be accessed.  Once a file is selected, that path becomes the current */
/* drive and path.  To change the drive or path, move the mouse to the  */
/* directory line of the selector box, click and backspace over the     */
/* line. Then click on the "closer" box of the selector.                */
/************************************************************************/

void
trunc_ext()
{
     WORD i;

     for(i=strlen(filepath);i>=0;--i){                                    
           if(filepath[i] == 92)    /* Get rid of the file extension. */
                break;              /* Eliminate characters back to the */
     }                              /* backslash character.             */
     filepath[i+1] = '\0';
}
   
          

openfile()
{
    if((fp = fopen(filepath,"r")) > 0) /* This is the ANSI defined function */  
        return(1);                   /* for level 2 files for Lattice C   */
     else{                           /* Alcyon uses file handles instead  */
           form_alert(1,alert4);     /* of file pointers.                 */
           return(0);
     } 
}



/**************************************************************************/
/*                           SCREEN OUTPUT                                */
/**************************************************************************/

void
put_labels()
{
      WORD namelen;

      lineaa();     /*  hide the mouse  */
      namelen = strlen(filepath);
      vst_effects(handle,1);
      v_gtext(handle,340-(namelen*5),30*rez,filepath);
      vst_effects(handle,0);
      v_gtext(handle,102,45*rez," Total letters : ");
      v_gtext(handle,86,61*rez," Total syllables : ");
      v_gtext(handle,420,109*rez," FOG Index Score : ");
      v_gtext(handle,119,77*rez," Total words : ");
      v_gtext(handle,86,93*rez," Total sentences : ");
      v_gtext(handle,78,109*rez," Avg letters/word : ");
      v_gtext(handle,410,141*rez," Flesch Index Score : ");
      v_gtext(handle,62,125*rez," Avg syllables/word : ");
      v_gtext(handle,62,141*rez," Avg words/sentence : ");
      v_gtext(handle,38,157*rez," Words with 1 syllable : ");
      v_gtext(handle,30,173*rez," Words with 2 syllables : ");
      v_gtext(handle,46,189*rez," Words with 3 or more : ");
      v_gtext(handle,380,45*rez," Character being read : ");
      v_gtext(handle,597,45*rez,"   ");
      linea9();   /*  show the mouse  */
}



put_str(n,field,decx)     /* based on itoa() and reverse() from K & R */
WORD field, decx;         /* A simpler approach would be to use the   */
LONG n;
{                         /*          sprintf(string,"%d,n);          */
        WORD i,j,c;       /* conversion of an integer to a string,but */        
                          /* my clib.bin file has a faulty sprintf    */
        i=0;              /* which causes a link failure.             */ 
        do{
             if(i == decx)       /* creates the string from the least */
                  string[i++] = '.';  /* significant digit to the most*/
             string[i++] = n % 10 + '0';
        }    while(( n /= 10) > 0);
        string[i] = '\0';
        for(i=0, j=strlen(string)-1; i<j; i++, j--){
             c = string[i];       /* so have to reverse the charcters. */
             string[i] = string[j];
             string[j] = c;    
        }
        if(decx == 0){
             j = strlen(string);
             string[j-1] = '\0';   /* get rid of the decimal point */
        }
        if(field<10)
             v_gtext(handle,260,rez*(field*16+45),&string);
        else
             v_gtext(handle,480,rez*(field*16-35),&string);
        return(1);
}




void
display_stats(let,wd,syl,sent,n3)
LONG let,wd,sent,syl,n3;
{
      LONG wps10,lpw10,spw10,fog10,flesch10;
 
      wps10 = div_round(wd*10,sent);
      lpw10 = div_round(let*10,wd);
      spw10 = div_round(syl*10,wd);
      put_str(wps10,6,1);
      put_str(lpw10,4,1);
      put_str(spw10,5,1);
      fog10 = div_round(400*n3,wd) + div_round(4*wd,sent);
      put_str(fog10,10,1);
      flesch10 = div_round(206835-(wps10*102)-(spw10*8460),100);
      put_str(flesch10,12,1);
      get_level(flesch10);    
}



void
get_level(level)  /* this is really 10X the Flesch index, but we're */
LONG level;                  /* scaling 10X for the 1 decimal place */
{
    if(level>900)
            v_gtext(handle,405,173*rez,flesch5);
    else {
         if(level>800)
            v_gtext(handle,405,173*rez,flesch6);
         else {
              if(level>700)
                   v_gtext(handle,405,173*rez,flesch7);
              else {
                   if(level>600)
                        v_gtext(handle,405,173*rez,flesch89);
                   else {
                        if(level>500)
                             v_gtext(handle,405,173*rez,flesch10);
                        else {
                             if(level>300)
                                  v_gtext(handle,405,173*rez,fleschc);
                             else v_gtext(handle,405,173*rez,fleschg);
                        }
                   }
               }
         }
    }
}



/*************************************************************************/
/*                          ANALYZING THE FILE                           */
/*************************************************************************/

void
process()
{
     WORD   c;
     WORD   i = 0;    
     LONG   numlet =0;     
     LONG   numsyl =0;     
     LONG   numwd =0;      
     LONG   numsent = 0;   
     LONG   syl;        
     LONG   num1 =0;       
     LONG   num2 =0;       
     LONG   num3 = 0;      
     char   cstr[2];

     cstr[1] = '\0';
     do{
          c= getc(fp);
          cstr[0] = c;
          v_gtext(handle,605,45*rez,cstr);
          if(isalpha(c)){
              numlet++; 
              put_str(numlet,0,0);       
              buf[i] =c;
              buf[i+1] = '\0';
              i++;
              v_gtext(handle,402,77*rez,buf); /* might want to guard against */
                                          /*overflow with a strlen qualifier */
          }   /* end of if... */
          else {
               if(istermpunc(c) && (buf[0] != '\0') && (strcmp(buf,"T"))){  
                      numsent++;                 /* because of *.DOC format */
                      put_str(numsent,3,0);    /* tab settings at beginning */
               } /* end of istermpunc()... */     /* of 1st Word and  */
               if(syl = parse()){               /* Wordwriter files */
                      numwd++;
                      put_str(numwd,2,0);
                      numsyl += syl; 
                      put_str(numsyl,1,0);
                      if(syl==1){
                            num1++;
                            put_str(num1,7,0);
                      }
                      if(syl==2){
                            num2++;
                            put_str(num2,8,0);
                      }
                      if(syl>2){
                            num3++;
                            put_str(num3,9,0);      
                      }      /* end of if(syl)... */
                 }   /* end of if(parse()...)... */
           i=0;
           strcpy(buf,"");
           v_gtext(handle,402,77*rez,"                        "); /*buf*/
           }    /* end of else ... */
     }while(c != EOF); 
     display_stats(numlet,numwd,numsyl,numsent,num3);
}


LONG
parse()
{                  
      LONG syl = 1;
      WORD i = 0;    /* points to beginning element for each cycle */
      WORD len;

      if(buf[0] == '\0'){
          return(0);
      }
      while((len = strlen(&buf[i])) > 2){   
          if(isvowel(buf[i]) && isconsonant(buf[i+1]) && isvowel(buf[i+2])) 
          {
               syl++;
          }
          else
          if(len>3){  
               if(isvowel(buf[i]) && isconsonant(buf[i+1]) 
                  && isconsonant(buf[i+2]) && isvowel(buf[i+3]))
               {
                     syl++;
                     i++;
               }
               if(isvowel(buf[i]) && isconsonant(buf[i+1]) 
                  && isconsonant(buf[i+2]) && isconsonant(buf[i+3]))
               {
                     syl++;
                     i+= 2;
               }
           }  /* end of if... */
           i++;
       } /* end of while... */
       return(syl);
}



istermpunc(c)
WORD c;
{
        if( c == '.' || c == '?' || c == '!' )
              return(1);
        else 
              return(0);
}



isvowel(ch)
WORD ch;
{
      WORD c;

      c = tolower(ch);
      if(c=='a' || c=='e' || c=='i' || c=='o' || c=='u' || c=='y')
           return(1);
      else
           return(0);
}

isconsonant(c)
WORD c;
{
      if(isalpha(c) && !(isvowel(c))) 
           return(1);
      else
           return(0);
}


LONG
div_round(n1,n2)          /* for integer division with rounding */
LONG n1,n2;
{
     LONG num1,num2;

     num1 = n1/n2;
     num2 = n1%n2;
     if((num2*2) > n2)
         return(num1 + 1);
     else
         return(num1);
}


                                                 
    
